//
//
//  RLPBiManager.m
//  RLPBi
//
//  Created by Brian Quan on 01/11/2012.
//  Copyright (c) 2012 Brian Quan. All rights reserved.
//

#import "RLPBiManager.h"
#import "RLPBiController.h"

#include "pbi_message_parse.h"

NSString *const RLPBiDidUpdateLocationNotification = @"RLPBiDidUpdateLocationNotification";

#define MESSAGE_LEN 58
#define KNOTS_TO_METER 1.94384449
#define ProtocolNMEA @"uk.co.racelogic.nmea"

@interface RLPBiManager (){
    RLPBiController *_eaSessionController;
    NSMutableData *_dataMutable;
}

@end

@implementation RLPBiManager

#pragma mark - Singleton
  static RLPBiManager *sessionController = nil;
+ (RLPBiManager *)sharedPBiManager{
    @synchronized(self)
    {
        if(sessionController==nil)
        {
            sessionController= [[RLPBiManager alloc] init];
        }
    }
    return sessionController;
}

#pragma mark - System function
- (id)init{
    if (! (self = [super init]))
        return nil;
    
    //Notifications
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_sessionDataReceived:) name:RLPBiSessionDataReceivedNotification object:nil];
    [[EAAccessoryManager sharedAccessoryManager] registerForLocalNotifications];
    
    [self checkCurrentAccessories];
    _eaSessionController = [RLPBiController sharedController];
    _dataMutable = [[NSMutableData alloc] init];
    
    return self;
}

#pragma mark - Self function
- (void)checkCurrentAccessories{
    
    NSMutableArray *listOfAvailableArray = [[NSMutableArray alloc] init];
    NSPredicate *bPredicate = [NSPredicate predicateWithFormat:@"SELF MATCHES \"uk.co.racelogic.nmea\""];
    for(EAAccessory *temp in [[EAAccessoryManager sharedAccessoryManager] connectedAccessories]){
        if ([[[temp protocolStrings] filteredArrayUsingPredicate:bPredicate] count] > 0) {
            [listOfAvailableArray addObject:temp];
        }
    }
    _selectedAccessory = listOfAvailableArray[0];
}

#pragma mark - Data receive
  static pbi_data_message_t dest;
- (void)_sessionDataReceived:(NSNotification *)notification{
    
    unsigned long bytesAvailable = 0;
    while ((bytesAvailable = [_eaSessionController readBytesAvailable]) > 0) {
        
        [_dataMutable appendData:[_eaSessionController readData:bytesAvailable]];
        if (_dataMutable) {
            while ([_dataMutable length] >= MESSAGE_LEN) {
                //Here is our buffer to parse
                unsigned char *buf = (unsigned char *)malloc(MESSAGE_LEN);
                memset(buf, 0, MESSAGE_LEN);
                
                //We put in only the length of one message, to avoid having the parser fail when more messages pile up
                memcpy(buf, [_dataMutable bytes], MESSAGE_LEN);
                
                //The parsing
                if (pbi_message_parse(&dest, buf, MESSAGE_LEN) == 0) {
                    /*
                     dest.sc_fields.num_satellites
                     dest.sc_fields.latitude
                     dest.sc_fields.longitude
                     dest.sc_fields.timestamp/1000.0
                     dest.sc_fields.speed/KNOTS_TO_METER
                     dest.sc_fields.heading
                     dest.sc_fields.height
                     ......
                     */
                    NSLog(@"\nSatellites number: %d\nLatitude: %f\nLogitude: %f\nTimestamp: %f\nSpeed: %f\nHeading: %f\nHeight: %f", dest.sc_fields.num_satellites, dest.sc_fields.latitude, dest.sc_fields.longitude, dest.sc_fields.timestamp/1000.0, dest.sc_fields.speed/KNOTS_TO_METER, dest.sc_fields.heading, dest.sc_fields.height);
                }
                free(buf);
                
                NSRange range = NSMakeRange(0, MESSAGE_LEN);
                [_dataMutable replaceBytesInRange:range withBytes:NULL length:0];
            }
        }
    }
    [[NSNotificationCenter defaultCenter] postNotificationName:RLPBiDidUpdateLocationNotification object:self userInfo:nil];
}

#pragma mark - Location
- (void)startUpdatingLocation{
    [_dataMutable setLength:0];
    [self sessionOpen];
}
- (void)stopUpdatingLocation{
    [self sessionClose];
}
- (void)sessionOpen{
    [_eaSessionController setupControllerForAccessory:_selectedAccessory withProtocolString:ProtocolNMEA];
    [_eaSessionController openSession];
}
- (void)sessionClose{
    [_eaSessionController closeSession];
}

@end