/*
 * pbi_message_parse.h
 *
 *  Created by Brian Quan on 01/11/2012.
 *  Copyright (c) 2012 Brian Quan. All rights reserved.
 */

#ifndef PBI_MESSAGE_PARSE_H_
#define PBI_MESSAGE_PARSE_H_

typedef char pbi_c;

typedef char pbi_s8;
typedef unsigned char pbi_u8;

typedef short int pbi_s16;
typedef short unsigned int pbi_u16;

typedef int pbi_s24;
typedef unsigned int pbi_u24;

typedef int pbi_s32;
typedef unsigned int pbi_u32;

typedef double pbi_scalar;

typedef struct pbi_header
{
	pbi_c header[8];
	pbi_u32 standard_channel_flags;
	pbi_u32 extended_channel_flags;
	pbi_c comma;
} pbi_header_t;

typedef struct pbi_sc_fields
{
	pbi_u8 num_satellites;
	pbi_u8 using_dgps;
	pbi_u32 timestamp;
	pbi_scalar latitude;
	pbi_scalar longitude;
	pbi_scalar speed;
	pbi_scalar heading;
	pbi_scalar height;
	pbi_scalar vertical_speed;
	pbi_scalar longitudinal_acceleration;
	pbi_scalar lateral_acceleration;
	pbi_scalar distance;
	pbi_scalar temperature;
	pbi_scalar media_free_space;
	pbi_u16 battery_voltage;
} pbi_sc_fields_t;

typedef struct pbi_ec_fields
{
	pbi_u16 battery_time_to_empty;
	pbi_u16 battery_voltage;
	pbi_u16 battery_charge_when_full;
	pbi_u16 battery_current_charge;
	pbi_u32 media_capacity;
	pbi_u32 media_free_space;
	pbi_scalar HDOP;
} pbi_ec_fields_t;

typedef struct pbi_data_message
{
	pbi_header_t header;
	pbi_sc_fields_t sc_fields;
	pbi_ec_fields_t ec_fields;
} pbi_data_message_t;

enum ESCFlags
{
	kSCFlags_Satellites                = 0x00000001,
	kSCFlags_Time                      = 0x00000002,
	kSCFlags_Latitude                  = 0x00000004,
	kSCFlags_Longitude                 = 0x00000008,
	kSCFlags_Speed                     = 0x00000010,
	kSCFlags_Heading                   = 0x00000020,
	kSCFlags_Height                    = 0x00000040,
	kSCFlags_VerticalSpeed             = 0x00000080,
	kSCFlags_LongitudinalAcceleration  = 0x00000100,
	kSCFlags_LateralAcceleration       = 0x00000200,
	kSCFlags_Distance                  = 0x00000800,
	kSCFlags_Temperature               = 0x02000000,
	kSCFlags_MediaFreeSpace            = 0x08000000,
	kSCFlags_BatteryVoltage            = 0x80000000,
};

enum EECFlags
{
	kECFlags_BatteryTimeToEmpty        = 0x00000001,
	kECFlags_BatteryVoltage            = 0x00000002,
	kECFlags_BatteryChargeWhenFull     = 0x00000004,
	kECFlags_BatteryCurrentCharge      = 0x00000008,
	kECFlags_MediaCapacity             = 0x00000010,
	kECFlags_MediaFreeSpace            = 0x00000020,
	kECFlags_HDOP                      = 0x00000040,
};

typedef enum
{
	kPbi_ok = 0,
	kPbi_err_checksum = -1,
	kPbi_err_header_too_short = -2,
	kPbi_err_unexpected_eos = -3,
	kPbi_err_data_format = -4,
	kPbi_err_invalid_buffer = -5,
} pbi_status;

pbi_status pbi_message_parse(pbi_data_message_t* dest, const unsigned char* raw_data, unsigned int size);

const char* pbi_error_message(pbi_status error_code);
int pbi_message_has_sc_field(const pbi_data_message_t* message, pbi_u32 sc_flags);
int pbi_message_has_ec_field(const pbi_data_message_t* message, pbi_u32 ec_flags);

#endif /* PBI_MESSAGE_PARSE_H_ */
